#include <arduino.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <EEPROM.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"
#include "DSCoreM_List.h"
#include "DSCoreM_DCC.h"
#include "DSCoreM_MM2.h"
#include "DSCoreM.h"
#include "sseg.h"


// Definition constance

#define DEBUG

#define BUTTON_DOWN 0b00000001
#define BUTTON_UP   0b00000010
#define BUTTON_F0   0b00000100
#define BUTTON_F1   0b00001000
#define BUTTON_F2   0b00010000
#define BUTTON_DIR  0b00100000

#define MODE_CV			1
#define MODE_LOC		2
#define MODE_INIT		3


#define WAIT_MSG 50


// Definition for variables
DSCoreLib Gateway;
SSEG SegDisplay(7, 8, A3, A4, 2, 3, 4, A5, A2);


//Loc mode
word gLocAddress = 3;
word eepLocAddress = 3;
byte gLocDirection = GO_FWD;
byte gLocSpeed = 0;

byte gButton_val = 0;
byte gLocFunc0 = 0;
byte gLocFunc1 = 0;
byte gLocFunc2 = 0;

byte gDisplayWordCount = 0;
byte gDisplayNext[2] = {SSEG_NONE, SSEG_NONE};

byte gMode_Main = 0;
byte gMode_Drive = 0;
byte gLCDCounter = 0;

//Last Speed
word gLastSpeed = 9999;

//Protcol
byte eepProtcol = 0;
word gLocProtcol = ADDR_DCC;
word gAccProtcol = ADDR_ACC_DCC;


//Acc mode
word gAccAddress = 1;


//CV mode
byte gMode_CV = 0;
byte gCVAddress = 1;
byte gCVValue = 0;


unsigned long gTime_5ms = 0;
unsigned long gTime_100ms = 0;
byte gDisplayCounter = 0;

word gPressCount = 0; /* */


// Functions
void displayModeCV();
void displayModeLoc();
void displayModeAcc();
void displayTemporary();
void processModeCV();
void processModeDial();
void processModeLoc();
byte getButtonStatus();
bool checkButton(byte inBuf, byte inCurrent, byte inButtonBit);
bool checkButtonLeaved(byte inBuf, byte inCurrent, byte inButtonBit);
bool checkButtonContinuous(byte inBuf, byte inCurrent, byte inButtonBit);
void saveEEPROM(byte inMode);
void loadEEPROM();
void updateEEPROM();

/*******************************
    Setup
********************************/

void setup()
{
	//Serial
	Serial.begin(115200);
	while (!Serial);

	Serial.println("--------------------------------------");
	Serial.println("Desktop Station Mint Can Box V2.0     ");
	Serial.println("--------------------------------------");
	Serial.println("100 Ready");
	
	//Load EEPROM
	loadEEPROM();
	
	
	Serial.print("Current Loc Addr: ");
	Serial.println(gLocAddress);
	
	//Analog input
	pinMode(A0, INPUT);
	digitalWrite(A0, LOW); // disable internal pullup
	analogReference(DEFAULT); // set analog reference 5V(VCC) 
	
	//Button
	pinMode(9, INPUT);
	pinMode(10, INPUT);
	pinMode(11, INPUT);
	pinMode(12, INPUT);
	pinMode(13, INPUT);
	pinMode(A1, INPUT);
	digitalWrite(9, HIGH);
	digitalWrite(10, HIGH);
	digitalWrite(11, HIGH);
	digitalWrite(12, HIGH);
	digitalWrite(13, HIGH);
	digitalWrite(A1, HIGH);
	
	
	//Set default button status
	gButton_val = getButtonStatus();
	
	//CV mode check
	
	if( ((gButton_val & BUTTON_F0) | (gButton_val & BUTTON_F1) | (gButton_val & BUTTON_F2)) > 0)
	{
		Serial.println("<<CV write mode>>");
		gMode_Main = MODE_CV;
		
	}
	else if((gButton_val & BUTTON_UP) > 0)
	{
		Serial.println("<<Change to DCC mode>>");
		/* Change to DCC mode */
		eepProtcol = 0;
		
		saveEEPROM(1);
		gMode_Main = MODE_INIT;
		
	}
	else if((gButton_val & BUTTON_DOWN) > 0)
	{
		Serial.println("<<Change to MM2 mode>>");
		/* Change to MM2 mode */
		eepProtcol = 1;
		
		saveEEPROM(1);
		gMode_Main = MODE_INIT;
		
	}
	else
	{
		Serial.println("<<Locomotive mode>>");
		gMode_Main = MODE_INIT;
		
	}
	
	/* Protcol changer */
	if( eepProtcol == 1)
	{
		/* MM2 Mode */
		Serial.println("MM2 Mode");
		gLocProtcol = ADDR_MM2;
		gAccProtcol = ADDR_ACC_MM2;
	}
	else
	{
		/* DCC Mode */
		Serial.println("DCC Mode");
		gLocProtcol = ADDR_DCC;
		gAccProtcol = ADDR_ACC_DCC + 1;
	}
	
	
	//Init Gateway
	Gateway.Init();
	
	//LCD
	
	switch( gMode_Main)
	{
	case MODE_CV:
		SegDisplay.SetCVNumber(gCVAddress);
		break;
		
	case MODE_INIT:
		if( eepProtcol == 1)
		{
			SegDisplay.SetWord(SSEG_M, SSEG_M);
		}
		else
		{
			SegDisplay.SetWord(SSEG_D, SSEG_C);
		}
		break;
	}
	
	SegDisplay.SetInit();

}

/*******************************
    Main loop
********************************/

void loop()
{
	//Scan
	if( (gLocAddress != 0) && (gMode_Main != MODE_CV))
	{
		//Power on
		if( Gateway.IsPower() == false)
		{
			Gateway.SetPower(true);
		}
		Gateway.Scan();
	}
	
	// 20ms interval
	if( (millis() - gTime_5ms) >= 10)
	{
		
		switch( gMode_Main)
		{
		case MODE_INIT:
			SegDisplay.ShowWord();
			break;

		case MODE_CV:
			
			displayModeCV();
			
			break;
			
		case MODE_LOC:
			
			if( gMode_Drive == 0)
			{
				displayModeLoc();
			}
			else
			{
				displayModeAcc();
			}
			
			
			break;
		}
		
		//Reset task
		gTime_5ms = millis();
		
	}
	
	// 100ms interval
	if( (millis() - gTime_100ms) >= 100)
	{
		switch( gMode_Main)
		{
		case MODE_INIT:
			
			if( gLCDCounter > 10 )
			{
				SegDisplay.SetNumber(gLocAddress);
				gMode_Main = MODE_LOC;
			}
			else
			{
				gLCDCounter++;
			}
			
			break;
			
		case MODE_CV:
			
			processModeCV();
			
			break;
			
		case MODE_LOC:
			
			//Update dial for speed control
			processModeDial();
			
			
			if( gMode_Drive == 0)
			{
				processModeLoc();
			}
			else
			{
				processModeAcc();
			}			
			
			break;
		}
		
		//Reset task
		gTime_100ms = millis();
	}
}

/*******************************
    DisplayModeCV
********************************/

void displayModeCV()
{
	//LCD control
	SegDisplay.ShowWord();
}

/*******************************
    DisplayModeLoc
********************************/

void displayModeLoc()
{
	//LCD control
	if( gDisplayWordCount > 0)
	{
		displayTemporary();
	}
	else
	{
		
		if( gLocAddress == 0)
		{
			
			gDisplayCounter++;
			
			if( gDisplayCounter <= 50)
			{
				SegDisplay.ShowNumber();
			}
			else if( gDisplayCounter <= 100)
			{
			
				SegDisplay.DisplayClear();
			}
			else
			{
				gDisplayCounter = 0;
			}
		}
		else
		{
			SegDisplay.ShowNumber();
		}
	}
}

/*******************************
    DisplayModeAcc
********************************/

void displayModeAcc()
{
	//LCD control
	if( gDisplayWordCount > 0)
	{
		displayTemporary();
	}
	else
	{
		SegDisplay.ShowWord();
	}
}

/*******************************
    DisplayTemporary
********************************/

void displayTemporary()
{
	//LCD control
	SegDisplay.ShowWord();
	gDisplayWordCount--;
	
	//Twice message function
	if( gDisplayWordCount == 0)
	{
		if ((gDisplayNext[0] == SSEG_NONE) && (gDisplayNext[1] == SSEG_NONE))
		{
			// do nothing
		}
		else
		{
			//Display Message
			SegDisplay.SetWord(gDisplayNext[0], gDisplayNext[1]);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_NONE;
			gDisplayNext[1] = SSEG_NONE;
		}
	}
}

/*******************************
    processModeCV
********************************/

void processModeCV()
{
	
	//Buttons
	byte aButton_val = getButtonStatus();
	
	if(aButton_val != gButton_val)
	{
		
		
		if(checkButton(gButton_val, aButton_val, BUTTON_UP))
		{
			
			switch( gMode_CV)
			{
			case 0:
				//CV No select
				//Increment CV no
				gCVAddress++;
				
				if(gCVAddress >= 10)
				{
					gCVAddress = 1;
				}
				
				//Display CV No.
				SegDisplay.SetCVNumber(gCVAddress);
				
				//DEBUG
				Serial.print("CV No: ");
				Serial.println(gCVAddress);
				break;
				
			case 1:
				//CV Value select
				//Increment CV no
				gCVValue++;
				
				if(gCVValue >= 100)
				{
					gCVValue = 0;
				}
				
				//Display CV Val.
				SegDisplay.SetCVValue(gCVValue);
				
				//DEBUG
				Serial.print("CV Value: ");
				Serial.println(gCVValue);
				break;
			}
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_DOWN))
		{
			
			switch( gMode_CV)
			{
			case 0:
				//CV No select
				//Decrement CV no
				
				if(gCVAddress == 1)
				{
					gCVAddress = 9;
				}
				else
				{
					//Decrement address
					gCVAddress--;
				}
				
				//Display CV No.
				SegDisplay.SetCVNumber(gCVAddress);
				
				//DEBUG
				Serial.print("CV No: ");
				Serial.println(gCVAddress);
				break;
				
			case 1:
				//CV Value select
				//Decrement CV no
				
				if(gCVValue == 0)
				{
					gCVValue = 99;
				}
				else
				{
					//Decrement address
					gCVValue--;
				}
				
				//Display CV Val.
				SegDisplay.SetCVValue(gCVValue);
				
				//DEBUG
				Serial.print("CV Value: ");
				Serial.println(gCVValue);
				break;
				
			}
			
		}
		else if (checkButton(gButton_val, aButton_val, BUTTON_F0))
		{
			//[hؑ
			gMode_CV++;
			
			switch( gMode_CV)
			{
			case 0:
				
				break;
				
			case 1:
				
				//Display CV Value.
				SegDisplay.SetCVValue(gCVValue);
				break;
				
			case 2:
				
				Serial.print("CV Write No=");
				Serial.print(gCVAddress);
				Serial.print(", Val=");
				Serial.println(gCVValue);
				Serial.println("CV Writing...");
				
				//CV Write
				Gateway.WriteConfig_Dir(gCVAddress, gCVValue);
				
				Serial.println("CV Write finished!");
				
				//Display Loc No.
				SegDisplay.SetCVNumber(gCVAddress);
				
				//Mode
				gMode_CV = 0;
				break;
				
			}
			
			//DEBUG
			
		}
		else if (checkButton(gButton_val, aButton_val, BUTTON_F2))
		{
			
			if( gMode_CV == 0)
			{
				//
			}
			else
			{
				//[hؑ
				gMode_CV--;
			}
			
			
			if( gMode_CV == 0)
			{
				//Display CV No.
				SegDisplay.SetCVNumber(gCVAddress);
			}
			
		}
		
		
		
		//Store to buf
		gButton_val = aButton_val;
		
	}
	else
	{
	}
	
}


/*******************************
    processModeDial
********************************/

void processModeDial()
{
	//Dial
	word aSpeed = (analogRead(A0) & 0b1111111100);
	
	if( ((aSpeed & 0b1111111100) != gLocSpeed) || ( gLocAddress == 0))
	{
		gLocSpeed = aSpeed & 0b1111111100;
		
		if( gLocAddress == 0)
		{
			int aSpeed_ana = (int)aSpeed - 16;
			
			if( aSpeed_ana < 0)
			{
				aSpeed_ana = 0;
			}
			
			if( gLocDirection == GO_FWD)
			{
				analogWrite(5, aSpeed >> 2);
				analogWrite(6, 0);
			}
			else
			{
				analogWrite(6, aSpeed >> 2);
				analogWrite(5, 0);
			}
		}
		else
		{
			analogWrite(6, 0);
			analogWrite(5, 0);
			
			Gateway.SetLocoSpeedEx(gLocProtcol + gLocAddress, aSpeed, 2);
			
		}
		
		//Update EEPROM
		updateEEPROM();
		
		//Debug
		if( gLastSpeed != aSpeed)
		{
			Serial.print("VOL: ");
			Serial.println(aSpeed);
			
			gLastSpeed = aSpeed;
		}
	}
}

/*******************************
    processModeLoc
********************************/

void processModeLoc()
{
	//Buttons
	byte aButton_val = getButtonStatus();
	
	if(aButton_val != gButton_val)
	{
		
		
		if(checkButton(gButton_val, aButton_val, BUTTON_UP))
		{
			//Increment address
			gLocAddress++;
			
			if(gLocAddress >= 100)
			{
				gLocAddress = 0;
			}
			
			//Display Loc No.
			SegDisplay.SetNumber(gLocAddress);
			
			//Reset press count
			gPressCount = 0;
			
			//DEBUG
			Serial.print("ADDR: ");
			Serial.println(gLocAddress);
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_DOWN))
		{
			
			if(gLocAddress == 0)
			{
				gLocAddress = 99;
			}
			else
			{
				//Decrement address
				gLocAddress--;
			}
			
			//Display Loc No.
			SegDisplay.SetNumber(gLocAddress);
			
			//Reset press count
			gPressCount = 0;
			
			//DEBUG
			Serial.print("ADDR: ");
			Serial.println(gLocAddress);
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F0))
		{
			
			//Reset press count
			gPressCount = 0;
			
		}
		else if(checkButtonLeaved(gButton_val, aButton_val, BUTTON_F0))
		{
			if( gLocAddress == 0)
			{
				return;
			}
			
			//DCC mode
			
			gLocFunc0 = (gLocFunc0 == 0) ? 1 : 0;
			
			Gateway.SetLocoFunction(gLocProtcol + gLocAddress, 0, gLocFunc0);
			
			//Reset press count
			gPressCount = 0;

			//Display Message
			SegDisplay.SetWord(SSEG_F, SSEG_0);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_O;
			gDisplayNext[1] = (gLocFunc0 == 0) ? SSEG_F : SSEG_N;
			
			//Update EEPROM
			updateEEPROM();

			//DEBUG
			Serial.print("F0: ");
			Serial.println(gLocFunc0);
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F1))
		{
			if( gLocAddress == 0)
			{
				return;
			}
			
			gLocFunc1 = (gLocFunc1 == 0) ? 1 : 0;
			
			Gateway.SetLocoFunction(gLocProtcol + gLocAddress, 1, gLocFunc1);
			
			//Display Message
			SegDisplay.SetWord(SSEG_F, SSEG_1);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_O;
			gDisplayNext[1] = (gLocFunc1 == 0) ? SSEG_F : SSEG_N;
			
			//Update EEPROM
			updateEEPROM();
			
			//DEBUG
			Serial.print("F1: ");
			Serial.println(gLocFunc1);
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F2))
		{
			if( gLocAddress == 0)
			{
				return;
			}
			
			gLocFunc2 = (gLocFunc2 == 0) ? 1 : 0;
			
			Gateway.SetLocoFunction(gLocProtcol + gLocAddress, 2, gLocFunc2);
			
			//Update EEPROM
			updateEEPROM();
			
			
			//Display Message
			SegDisplay.SetWord(SSEG_F, SSEG_2);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_O;
			gDisplayNext[1] = (gLocFunc2 == 0) ? SSEG_F : SSEG_N;
			
			//DEBUG
			Serial.print("F2: ");
			Serial.println(gLocFunc2);
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_DIR))
		{
			
			gLocDirection = (gLocDirection == GO_FWD) ? GO_REV : GO_FWD;
			
			Gateway.SetLocoDirection(gLocProtcol + gLocAddress, gLocDirection);
			
			//Display Message
			if( gLocDirection == GO_FWD)
			{
				SegDisplay.SetWord(SSEG_F, SSEG_D);
			}
			else
			{
				SegDisplay.SetWord(SSEG_R, SSEG_V);
			}
			gDisplayWordCount = WAIT_MSG;
			
			//Update EEPROM
			updateEEPROM();
			
			//DEBUG
			Serial.print("DIR: ");
			Serial.println((gLocDirection == GO_FWD) ? "FWD" : "REV");
		}
		
		
		
		//Store to buf
		gButton_val = aButton_val;
		
	}
	else
	{
		//Buttons have no difference from previous state
		
		if(checkButtonContinuous(gButton_val, aButton_val, BUTTON_UP))
		{
			if( gPressCount > 15)
			{
				//Increment address
				gLocAddress++;
				
				if(gLocAddress >= 100)
				{
					gLocAddress = 0;
				}
				
				//Display Loc No.
				SegDisplay.SetNumber(gLocAddress);
			}
			else
			{
				gPressCount++;
			}
		}
		else if(checkButtonContinuous(gButton_val, aButton_val, BUTTON_DOWN))
		{
			if( gPressCount > 15)
			{
				if(gLocAddress == 0)
				{
					gLocAddress = 99;
				}
				else
				{
					//Decrement address
					gLocAddress--;
				}
				
				//Display Loc No.
				SegDisplay.SetNumber(gLocAddress);
			}
			else
			{
				gPressCount++;
			}
		}
		
		else if(checkButtonContinuous(gButton_val, aButton_val, BUTTON_F0))
		{
			if( gPressCount > 15)
			{
				gMode_Drive = 1;
				
				//Display Acc No.
				SegDisplay.SetAccessory(gAccAddress);
			}
			else
			{
				if(gLocAddress != 0)
				{
					gPressCount++;
				}
				
			}
		}		
		
	}
	
}

/*******************************
    processModeAcc
********************************/

void processModeAcc()
{
	//Buttons
	byte aButton_val = getButtonStatus();
	
	if(aButton_val != gButton_val)
	{
		
		
		if(checkButton(gButton_val, aButton_val, BUTTON_UP))
		{
			//Increment address
			gAccAddress++;
			
			if(gAccAddress >= 10)
			{
				gAccAddress = 1;
			}
			
			//Display Acc No.
			SegDisplay.SetAccessory(gAccAddress);
			
			//DEBUG
			Serial.print("ACC ADDR: ");
			Serial.println(gAccAddress);
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_DOWN))
		{
			
			if(gAccAddress == 1)
			{
				gAccAddress = 9;
			}
			else
			{
				//Decrement address
				gAccAddress--;
			}
			
			//Display Acc No.
			SegDisplay.SetAccessory(gAccAddress);
			
			//DEBUG
			Serial.print("ACC ADDR: ");
			Serial.println(gAccAddress);
			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F0))
		{
			//Diverse
			
			if( gAccAddress == 0)
			{
				return;
			}
			
			//DCC mode
			
			Gateway.SetTurnout(gAccProtcol + gAccAddress, 0);
			
			//Display Message
			SegDisplay.SetWord(SSEG_MINUS, SSEG_MINUS);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_A;
			gDisplayNext[1] = gAccAddress;
			
			//Update EEPROM
			updateEEPROM();

			//DEBUG
			Serial.print("Turnout Diverse(/) ");
			Serial.println(gAccAddress);			
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F1))
		{
			//Straight
			
			if( gAccAddress == 0)
			{
				return;
			}
			
			Gateway.SetTurnout(gAccProtcol + gAccAddress, 1);
			
			//Display Message
			SegDisplay.SetWord(SSEG_I, SSEG_I);
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_A;
			gDisplayNext[1] = gAccAddress;
			
			//DEBUG
			Serial.print("Turnout Straight(|) ");
			Serial.println(gAccAddress);
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_F2))
		{
			//Go to Loc mode
			
			gMode_Drive = 0;
			
			//Display Loc No.
			SegDisplay.SetNumber(gLocAddress);
			
			
			//DEBUG
			Serial.println("Loc mode");
		}
		else if(checkButton(gButton_val, aButton_val, BUTTON_DIR))
		{
			
			gLocDirection = (gLocDirection == GO_FWD) ? GO_REV : GO_FWD;
			
			Gateway.SetLocoDirection(gLocProtcol + gLocAddress, gLocDirection);
			
			//Display Message
			if( gLocDirection == GO_FWD)
			{
				SegDisplay.SetWord(SSEG_F, SSEG_D);
			}
			else
			{
				SegDisplay.SetWord(SSEG_R, SSEG_V);
			}
			gDisplayWordCount = WAIT_MSG;
			gDisplayNext[0] = SSEG_A;
			gDisplayNext[1] = gAccAddress;
			
			//Update EEPROM
			updateEEPROM();
			
			//DEBUG
			Serial.print("DIR: ");
			Serial.println((gLocDirection == GO_FWD) ? "FWD" : "REV");
		}
		
		
		
		//Store to buf
		gButton_val = aButton_val;
		
	}
	else
	{
		//Buttons have no difference from previous state
	}
	
}




/*******************************
    getButtonStatus
********************************/

byte getButtonStatus()
{
	
	byte aButton_val = 0;
	
	//Input Buttons
	bitWrite(aButton_val, 0, digitalRead(9)); // Down
	bitWrite(aButton_val, 1, digitalRead(10));// Up
	bitWrite(aButton_val, 2, digitalRead(11));// F0
	bitWrite(aButton_val, 3, digitalRead(12));// F1
	bitWrite(aButton_val, 4, digitalRead(13));// F2
	bitWrite(aButton_val, 5, (digitalRead(A1) == 1) ? 0 : 1);// DIR
	
	return ~aButton_val;
	
}

/*******************************
    checkButton
********************************/

bool checkButton(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( ((inBuf & inButtonBit) == 0) && ((inCurrent & inButtonBit) > 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

/*******************************
    checkButtonLeaved
********************************/

bool checkButtonLeaved(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( ((inBuf & inButtonBit) > 0) && ((inCurrent & inButtonBit) == 0))
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

/*******************************
    checkButtonContinuous
********************************/

bool checkButtonContinuous(byte inBuf, byte inCurrent, byte inButtonBit)
{

	if( (inCurrent & inButtonBit) > 0)
	{
		return true;
	}
	else
	{
		return false;
	}
	
}

/*******************************
    Update EEPROM
********************************/

void updateEEPROM()
{
	if( gLocAddress != eepLocAddress)
	{
		//Update Loc Addr EEPROM
		saveEEPROM(0);
		
		//Update buffer
		eepLocAddress = gLocAddress;
	}
	
}

/*******************************
    Load EEPROM
********************************/

void loadEEPROM()
{
	
	if( EEPROM.read(0) == 0xCC)
	{
		
		eepLocAddress = (EEPROM.read(3) << 8) + EEPROM.read(2);
		eepProtcol = EEPROM.read(4);
		gLocAddress = eepLocAddress;
		
		if( eepProtcol > 1)
		{
			eepProtcol = 0;
		}
		
	}
	
}

/*******************************
    Save EEPROM
********************************/

void saveEEPROM(byte inMode)
{
	
	if( EEPROM.read(0) == 0xCC)
	{
		
		if( inMode == 0)
		{
			EEPROM.write(2, lowByte(gLocAddress));
			EEPROM.write(3, highByte(gLocAddress));
			
		}
		
		if( inMode == 1)
		{
			EEPROM.write(4, eepProtcol);
			
		}		
	}
	else
	{
		//Mint can ID
		EEPROM.write(0, 0xCC);
		
		//Data write
		EEPROM.write(1, 0);
		EEPROM.write(2, lowByte(gLocAddress));
		EEPROM.write(3, highByte(gLocAddress));
		EEPROM.write(4, eepProtcol);
		
	}
	
}



